// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: LayerGrid.java
// Classes: LayerGrid
// Original Author: jrobbins@ics.uci.edu
// $Id: LayerGrid.java,v 1.13 1997/01/24 19:20:30 jrobbins Exp $

package uci.graphedit;

import java.util.*;
import java.awt.*;


/** Draw a background drawing guide consisting of horizontal and
 * vertical lines in a neutral color. This feature is common to many
 * drawing applications (e.g., MacDraw). LayerGrid is in concept a
 * Layer, just like any other so it can be composed, locked, grayed,
 * hidden, and reordered. <p>
 *
 * Needs-More-Work: since I am using image stamping and AWT does not
 * support transparent pixels, any layer underneath an instance of
 * LayerGrid will not be seen. Image stamping is nice because it is
 * fast, but I need to think of some other fast way to do it... */

public class LayerGrid extends Layer {
  
  /** The spacing between the lines */
  private int _spacing = 16;

  /** True means draw grid lines, false means draw only dots where the
  * lines would intersect. Drawing dots is about as useful as drawing
  * lines and it looks less cluttered. But lines are more familiar to
  * some people. */
  private boolean _drawLines = true;

  /** The image stamp is used to draw the grid quickly. Basically I
   * make a pretty big off screen image and draw my grid on it, then I
   * repeatedly bitblt that image to the screen (or some other off
   * screen image). More pixels are being drawn this way, but since
   * bitblt is a fast operation it works much faster. */
  transient private Image _stamp = null;
  private int _stampWidth = 128, _stampHeight = 128;

  /** The color of the grid lines or dots */
  protected Color _color = Color.gray;
  //(155, 155, 155);

  /** the color of the space between the lines or
   * dots. Needs-More-Work: Ideally this would be transparent, but AWT
   * does not fully support transparent pixels yet. */
  protected Color _bgColor = Color.lightGray;
  //  (+ 192 192 192) (+ 160 160 250)


  /** As an example of different grid styles 5 are defined
   *
   * @see LayerGrid#adjust
   */
  private int _style = 0;
  private final int NUM_STYLES = 5;
  
  /** Construct a new LayerGrid and name it 'Grid' */
  public LayerGrid() { super("Grid"); }

  /** Construct a new LayerGrid with the given foreground color,
   * background color, line spacing, and lines/dots flag */
  public LayerGrid(Color fore, Color back, int spacing, boolean lines ) {
    super("Grid");
    _color = fore;
    _bgColor = back;
    _spacing = spacing;
    _drawLines = lines;
  }

  /** Draw the grid lines or dots by repeatedly bitblting a
   * precomputed 'stamp' onto the given Graphics */
  public synchronized void drawContents(Graphics g) {
    if (_stamp == null) {
      System.out.println("making stamp");
      if (_spacing > _stampHeight) 
	_stampHeight = _stampWidth = _spacing;
      Editor ce = Globals.curEditor();
      if (ce == null) {
	// this is a bad idea, but it works around a very awkward AWT
	// requirement: that only frames can make Image instances
	System.out.println("no editor");
	Frame f = new Frame(); f.show();
	_stamp = f.createImage(_stampWidth, _stampHeight);
	f.dispose();
      }
      else { System.out.println("editor is making image");
	_stamp = ce.createImage(_stampWidth, _stampHeight);
      }	
      if (_stamp == null) System.out.println("could not make stamp!");
      if (_drawLines) drawLines(_stamp);
      else drawDots(_stamp);
    }
      
    Rectangle clip = g.getClipRect();
    int x = clip.x / _spacing * _spacing;
    int y = clip.y / _spacing * _spacing;
    int bot = clip.y + clip.height;
    int right = clip.x + clip.width;

    while (x <= right) {
      y = clip.y / _spacing * _spacing;
      while (y <= bot) {
	g.drawImage(_stamp, x, y, null);
	y += _stampHeight;
      }
      x += _stampWidth;
    }    
  }

  /** Draw lines on the given stamp Image */
  private void drawLines(Image i) {
    Graphics g = i.getGraphics();
    g.setColor(_bgColor);
    g.fillRect(0, 0, _stampWidth, _stampHeight);
    int x = 0;
    int y = 0;
    int stepsX = _stampWidth / _spacing + 2;
    int stepsY = _stampHeight / _spacing + 2;
    int right = _stampWidth;
    int bot = _stampHeight;
    g.setColor(_color);

    while (stepsX > 0) {
      g.drawLine(x, 0, x, bot);
      x += _spacing;
      --stepsX;
    }
    while (stepsY > 0) {
      g.drawLine(0, y, right, y);
      y += _spacing;
      --stepsY;
    }
  }

  /** Draw dots on the given stamp Image */
  protected void drawDots(Image i) {
    Graphics g = i.getGraphics();
    g.setColor(_bgColor);
    g.fillRect(0, 0, _stampWidth, _stampHeight);
    int x = 0;
    int y = 0;
    int right = _stampWidth;
    int bot = _stampHeight;

    g.setColor(_color);
    while (x <= right) {
      y = 0;
      while (y <= bot) {
	g.fillRect(x, y, 2, 2);
	y += _spacing;
      }
      x += _spacing;
    }
  }
      
  /** Eventually this will open a dialog box to let the user adjust
   * the grid line spacing, colors, and whether liens or dots are
   * shown. For now it just cycles among 5 predefined styles. */
  public void adjust() {
    _style = (_style + 1) % NUM_STYLES;
    _stamp = null;
    switch (_style) {
    case 0:
      hidden(false); _drawLines = true; _spacing = 16;
      break;
    case 1:
      hidden(false); _drawLines = true; _spacing = 8;
      break;
    case 2:
      hidden(false); _drawLines = false; _spacing = 16;
      break;
    case 3:
      hidden(false); _drawLines = false; _spacing = 32;
      break;
    case 4:
      hidden(true);
      break;
    }
    setChanged();
    Vector v = new Vector(2);
    v.addElement("refresh");
    v.addElement(this);
    notifyObservers(v);    
  }

} /* end class LayerGrid */
